package minecraft.gui.mvp;


import minecraft.player.InvalidCharacterParamsException;
import minecraft.player.Steve;

import javax.swing.*;
import java.util.List;

public class StevePresenter implements SteveView.SteveViewListener {
    private final SteveView view;
    private final SteveRepository repository;

    public StevePresenter(SteveView view, SteveRepository repository) {
        this.view = view;
        this.repository = repository;
        this.view.setListener(this);
        loadAllSteves();
    }

    private void loadAllSteves() {
        new SwingWorker<List<Steve>, Void>() {
            @Override
            protected List<Steve> doInBackground() throws Exception {
                // Simulate a long loading process
                Thread.sleep(1000);
                return repository.getAllSteves();
            }

            @Override
            protected void done() {
                try {
                    view.displayAllSteves(get());
                } catch (Exception e) {
                    view.displayError("Error loading Steves: " + e.getMessage());
                }
            }
        }.execute();
    }

    @Override
    public void onAddSteve() {
        try {
            final Steve steve = createSteveFromForm();

            new SwingWorker<Void, Void>() {
                @Override
                protected Void doInBackground() throws Exception {
                    repository.addSteve(steve);
                    // Simulate a long save operation
                    Thread.sleep(500);
                    return null;
                }

                @Override
                protected void done() {
                    try {
                        get();
                        loadAllSteves();
                        view.clearForm();
                        view.displaySuccess("Steve added successfully!");
                    } catch (Exception e) {
                        view.displayError("Error saving Steve: " + e.getMessage());
                    }
                }
            }.execute();

        } catch (InvalidCharacterParamsException e) {
            view.displayError(e.getMessage());
        }
    }

    @Override
    public void onUpdateSteve() {
        int selectedIndex = view.getSelectedIndex();
        if (selectedIndex < 0) {
            view.displayError("Please select a Steve to update");
            return;
        }

        try {
            Steve updatedSteve = createSteveFromForm();
            repository.updateSteve(selectedIndex, updatedSteve);
            view.displayAllSteves(repository.getAllSteves());
            view.displaySuccess("Steve updated successfully!");
        } catch (InvalidCharacterParamsException e) {
            view.displayError(e.getMessage());
        }
    }

    @Override
    public void onDeleteSteve() {
        int selectedIndex = view.getSelectedIndex();
        if (selectedIndex < 0) {
            view.displayError("Please select a Steve to delete");
            return;
        }

        int confirm = JOptionPane.showConfirmDialog(
                (JFrame) view,
                "Are you sure you want to delete this Steve?",
                "Confirm Delete",
                JOptionPane.YES_NO_OPTION
        );

        if (confirm == JOptionPane.YES_OPTION) {
            repository.removeSteve(selectedIndex);
            view.displayAllSteves(repository.getAllSteves());
            view.clearForm();
            view.displaySuccess("Steve deleted successfully!");
        }
    }

    @Override
    public void onClearForm() {
        view.clearForm();
    }

    @Override
    public void onSteveSelected(int index) {
        if (index >= 0 && index < repository.getAllSteves().size()) {
            view.displaySteveDetails(repository.getSteve(index));
        }
    }

    @Override
    public void onExportAllSteves(String filename) {
        view.enableExportButton(false);
        view.showProgress();

        new SwingWorker<Void, Integer>() {
            @Override
            protected Void doInBackground() throws Exception {
                List<Steve> steves = repository.getAllSteves();
                int total = steves.size();

                for (int i = 0; i < total; i++) {
                    Thread.sleep(1000);
                    int progress = (i * 100) / total;
                    publish(progress);
                    // TODO actual export!
                }
                return null;
            }

            @Override
            protected void process(List<Integer> chunks) {
                view.updateProgress(chunks.get(chunks.size() - 1));
            }

            @Override
            protected void done() {
                try {
                    get(); // blokovanie
                    view.displaySuccess("All Steves exported successfully to: " + filename);
                } catch (Exception e) {
                    view.displayError("Export failed: " + e.getMessage());
                } finally {
                    view.hideProgress();
                    view.enableExportButton(true);
                }
            }
        }.execute();
    }

    private Steve createSteveFromForm() throws InvalidCharacterParamsException {
        return new Steve(
                view.getName(),
                view.getHealth(),
                view.getMaxHealth(),
                view.getAttackPower(),
                view.getDefensePower(),
                view.getHunger(),
                view.getMaxHunger(),
                view.isGodMode()
        );
    }
}
